# Test script for Perl extension Curl::easy.
# Check out the file README for more info.

# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..6\n"; }
END {print "ko 1\n" unless $loaded;}
use Curl::easy;
$loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

sub prog_callb
{
    print "\nperl progress_callback has been called!\n";
    print "clientp: $_[0], dltotal: $_[1], dlnow: $_[2], ultotal: $_[3], ";
    print "ulnow: $_[4].\n";
    return 0;
}

sub write_callb
{
    print "\nperl write_callback has been called!\n";
    print "data: $_[0], size: $_[1], num: $_[2].\n";
    return $_[1] * $_[2];
}

sub read_callb
{
    print "\nperl read_callback has been called!\n";
    print "data: $_[0], size: $_[1], num: $_[2].\n";
    print "\nEnter max ", $_[1] * $_[2], " characters to be uploaded.\n";
    print "(End with Ctrl-D): ";
    $_[0] = <STDIN>;
    return length($_[0]);
}

sub head_callb
{
    print "\nperl header_callback has been called!\n";
    print "data: $_[0], size: $_[1], num: $_[2].\n";
    return $_[1] * $_[2];
}

# First, do an http download:

# Read URL to get
$defurl = "http://localhost/cgi-bin/printenv";
$url = "";
print "Please enter an URL for http download [$defurl]: ";
$url = <STDIN>;
if ($url =~ /^\s*\n/) {
    $url = $defurl;
}

# Init the curl session
if (($curl = Curl::easy::init()) != 0) {
    print "ok 2\n";
} else {
    print "ko 2\n";
}

# Set URL to get
if (Curl::easy::setopt($curl, Curl::easy::CURLOPT_URL, $url) == 0) {
    print "ok 3\n";
} else {
    print "ko 3\n";
}

# No progress meter please
#Curl::easy::setopt($curl, Curl::easy::CURLOPT_NOPROGRESS, 1);

# Use our own progress callback
Curl::easy::setopt($curl, Curl::easy::CURLOPT_PROGRESSFUNCTION, \&prog_callb);

# Follow location headers
Curl::easy::setopt($curl, Curl::easy::CURLOPT_FOLLOWLOCATION, 1);

# Set file where to read cookies from
Curl::easy::setopt($curl, Curl::easy::CURLOPT_COOKIEFILE, "cookies");
Curl::easy::setopt($curl, Curl::easy::CURLOPT_COOKIE, undef);

# Store error messages in $errbuf
$errbuf = "";
Curl::easy::setopt($curl, Curl::easy::CURLOPT_ERRORBUFFER, "errbuf");

# Use the following to store headers and/or content into files:
# Set file where to store the headers
#open HEAD, ">head.out";
#Curl::easy::setopt($curl, Curl::easy::CURLOPT_WRITEHEADER, HEAD);

# Set file where to store the content
#open BODY, ">body.out";
#Curl::easy::setopt($curl, Curl::easy::CURLOPT_FILE, BODY);

# Store headers and content into internal variables, instead of writing
# them to a file. Headers are stored into $Curl::easy::headers, content
# into $Curl::easy::content
#Curl::easy::internal_setopt(Curl::easy::USE_INTERNAL_VARS, 1);

# Use perl write callback to store retrieved data
Curl::easy::setopt($curl, Curl::easy::CURLOPT_WRITEFUNCTION, \&write_callb);

# Store the headers also
#Curl::easy::setopt($curl, Curl::easy::CURLOPT_HEADER, 1);

# Use perl header callback to store header data
Curl::easy::setopt($curl, Curl::easy::CURLOPT_HEADERFUNCTION, \&head_callb);

# Add some additional headers to the http-request:
$myheaders[0] = "Server: localhost";
$myheaders[1] = "User-Agent: Perl interface for libcURL";
$myheaders[2] = "";
Curl::easy::setopt($curl, Curl::easy::CURLOPT_HTTPHEADER, \@myheaders);

print "ok 4\n";

# Go get it
if (Curl::easy::perform($curl) == 0) {
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_SIZE_DOWNLOAD, $bytes);
    print "ok 5: $bytes bytes read\n\n";
    #print "Headers:\n$Curl::easy::headers\n";
    #print "Content:\n$Curl::easy::content\n";
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_EFFECTIVE_URL, $realurl);
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_HTTP_CODE, $httpcode);
    print "Effective fetched url (http code: $httpcode) was: $realurl\n";
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_TOTAL_TIME, $total);
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_NAMELOOKUP_TIME, $dns);
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_CONNECT_TIME, $conn);
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_PRETRANSFER_TIME, $pre);
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_STARTTRANSFER_TIME, $start);
    print "times are: dns: $dns, connect: $conn, pretransfer: $pre, starttransfer: $start, total: $total.\n";
} else {
    # We can acces the error message in $errbuf here
    print "ko 5: '$errbuf'\n";
}

# Cleanup
Curl::easy::cleanup($curl);
print "ok 6\n";


# Now do an ftp upload:

$defurl = "ftp://localhost//tmp/bla";
print "\n\nPlease enter an URL for ftp upload [$defurl]: ";
$url = <STDIN>;
if ($url =~ /^\s*\n/) {
    $url = $defurl;
}

print "\n\nPlease enter username and passwort in the form user:pass: ";
$user_pwd = <STDIN>;

# Init the curl session
if (($curl = Curl::easy::init()) != 0) {
    print "ok 2\n";
} else {
    print "ko 2\n";
}

# Set URL to get
if (Curl::easy::setopt($curl, Curl::easy::CURLOPT_URL, $url) == 0) {
    print "ok 3\n";
} else {
    print "ko 3\n";
}

# Set user and pwd to use:
Curl::easy::setopt($curl, Curl::easy::CURLOPT_USERPWD, $user_pwd);

# Tell libcurl to to an upload
Curl::easy::setopt($curl, Curl::easy::CURLOPT_UPLOAD, 1);

# No progress meter please
#Curl::easy::setopt($curl, Curl::easy::CURLOPT_NOPROGRESS, 1);

# Use our own progress callback
Curl::easy::setopt($curl, Curl::easy::CURLOPT_PROGRESSFUNCTION, \&prog_callb);

# Store error messages in $errbuf
Curl::easy::setopt($curl, Curl::easy::CURLOPT_ERRORBUFFER, "errbuf");

# Use perl read callback to read data to be uploaded
Curl::easy::setopt($curl, Curl::easy::CURLOPT_READFUNCTION,
    \&read_callb);

print "ok 4\n";

# Go send it
if (Curl::easy::perform($curl) == 0) {
    Curl::easy::getinfo($curl, Curl::easy::CURLINFO_SIZE_UPLOAD, $bytes);
    print "ok 5: $bytes bytes transferred\n\n";
} else {
    # We can acces the error message in $errbuf here
    print "ko 5: '$errbuf'\n";
}

# Cleanup
Curl::easy::cleanup($curl);
print "ok 6\n";

